/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University. 
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Created by: XIAO Tong (xiaotong@mail.neu.edu.cn) 2018-07-31
 */

#include "T2TAttention.h"
#include "T2TUtility.h"

namespace transformer
{

/* constructor */
T2TAttention::T2TAttention()
{
    nhead = -1;
    dk = -1;
    dv = -1;
    d  = -1;
}

/* deconstructor */
T2TAttention::~T2TAttention()
{
}

/* 
initialize the model 
>> argc - number of arguments
>> argv - list pf pointers to the arguments
>> devID - device id
>> mem - the memory pool
*/
void T2TAttention::InitModel(int argc, const char ** argv, int devID, XMem * mem)
{
    float minmax = 0;

    LoadParamInt(argc, argv, "nhead", &nhead, 8);
    LoadParamInt(argc, argv, "dk", &dk, 512);
    LoadParamInt(argc, argv, "dv", &dv, 512);
    LoadParamInt(argc, argv, "d", &d, 512);
    LoadParamFloat(argc, argv, "attminmax", &minmax, 0.08F);

    InitTensor2D(&w, 3 * d, 2 * dk + dv, X_FLOAT, devID, mem);

    w.SetDataRand(-minmax, minmax);
}

}